var UniqueID = 0;

var SVG_STYLES = [
    'background-color',
    'display',
    'font-family',
    'font-size',
    'fill',
    'stroke',
    'stroke-width',
    'stroke-dasharray'
]

function getUniqueID() {
	return UniqueID++;
}

function DrawMindGraph(pic, force_scale, max_height, has_descs, show_descs, force_expand) {
    var data = pic.find('.sentence-pic-data');
    var body = pic.find('.sentence-pic-body');
    var options = {
        'tokens': JSON.parse(data.find('tokens').text()),
        'phrasals': JSON.parse(data.find('phrasals').text()),
        'components': JSON.parse(data.find('components').text()),
        'chart': JSON.parse(data.find('chart').text()),
        'has_watermark': false,
        'platform':'browser'
    }
    if (has_descs) {
        var descs = data.find('descs').text();
        if (descs) {
            options['descs'] = JSON.parse(descs);
        }
        else {
            has_descs = false;
            show_descs = false;
        }
    }
    if (force_scale) {
        options['force_scale'] = force_scale;
    }
    if (max_height) {
        options['max_height'] = max_height;
    }
    if (has_descs) {
        options['has_descs'] = has_descs;
    }
    if (show_descs) {
        options['show_descs'] = show_descs;
    }
    if (force_expand) {
        options['force_expand'] = force_expand;
    }
    body.empty();
    var mind_chart = body.DrawMindChart(options);
    return mind_chart;
}

(function($){
"use strict";
$.fn.postData = async function(url, data, max_try_count, with_error) {
	this.try_count = 0;
	if (max_try_count == undefined) {
		max_try_count = 1;
	}
	else if (max_try_count == 0) {
		max_try_count = 30;
	}

	var that = this;
	async function postData(wrap, url, data) {
		try {
			var result = await $.ajax({
				type: "POST",
				url: url,
				data: data,
				dataType: 'json',
				time: 5000,
				crossDomain: true,
				xhrFields: { withCredentials: true },
				beforeSend: function(xhr, options) {
					xhr.setRequestHeader('X-PJAX', 'true');
					xhr.setRequestHeader('X-Requested-With', 'XMLHttpRequest');
				}
			})
		}
		catch(response) {
			if (response.status == 403) {
				wrap.showBlockAlert('当前会话已失效，请刷新页面', 'error');
				throw '';
			}
			if (max_try_count == 0 || that.try_count >= max_try_count) {
				wrap.showBlockAlert('与后台服务通信时出现异常，请检查网络后再次尝试', 'error');
				throw '';
			}
			that.try_count += 1;
			await delay(that.try_count * 200);
			var result = await postData(wrap, url, data);
			return result;
		}
		var status = result['status'];
		if (status == 'timeout') {
			that.try_count += 1;
			await delay(that.try_count * 200);
			result = await postData(wrap, url, data);
			return result;
		}
		else if (status != 'success') {
			wrap.showBlockAlert(result['error_type'], 'warning');
            if (!with_error) {
			    throw result['error_type'];
            }
		}
		return result;
	}

	function delay(ms) {
		return new Promise(resolve => setTimeout(resolve, ms));
	}

	var result = await postData($(this), url, data);
	return result;
}
})(jQuery);

// ------------------------------------------- Block Alert -------------------------------------------------
(function($){
    "use strict";

    var error_node =$('<div>');
    error_node.append(
        $('<i>')
    ).append(
        $('<span>', {'class': 'error-text'})
    )
    
    $.fn.showBlockAlert = function(error_text, error_type) {
        if (this.showing) {
            return;
        }
        this.showing = true;
        error_node.find('span').html(error_text);
        var block = $(this);
        if ($(this).hasClass('modal') || $(this).hasClass('modal-dialog')) {
            block = $(this).find('.modal-content');
        }
        var top_offset = 0;
        if ($(this)[0].tagName == 'ION-HEADER') {
            top_offset = 30;
        }
        var fadeTimeout;
        var class_names = 'error-node p-3 rounded-3 translate-middle';
        if ($(this).hasClass('full-screen')) {
            class_names += ' full-screen';
        }
        if (error_type == 'success') {
            error_node.attr('class', class_names + ' text-bg-success');
            error_node.find('i').attr('class', 'fa fa-check me-2');
            fadeTimeout = 1500;
        }
        else if (error_type == 'warning') {
            error_node.attr('class', class_names + ' text-bg-warning');
            error_node.find('i').attr('class', 'fa fa-info-circle me-2');
            fadeTimeout = 3000;
        }
        else if (error_type == 'error') {
            error_node.attr('class', class_names + ' text-bg-danger');
            error_node.find('i').attr('class', 'fa fa-warning me-2');
            fadeTimeout = 4000;
        }
        else {
            return;
        }
        block.append(error_node);
        error_node.animate({top: (top_offset + 16) + 'px', opacity: 1}, 400);
        var that = this;
        setTimeout(function() {
            error_node.animate({top: (top_offset - 48) + 'px', opacity: 0}, 400);
            that.showing = false;
        }, fadeTimeout);
    }

    $.fn.showFixedAlert = function(alert_title, alert_content) {
        var alert = $('<div>', {'class': 'alert alert-warning alert-dismissible fade show'});
        var title = $('<div>').html(alert_title);
        alert.append(title);
        if (alert_content) {
            var content = $('<div>').html(alert_content);
            alert.append(content);
        }
        var dismiss_btn = $('<button>', {
            'type': 'button',
            'class': 'btn-close',
            'data-bs-dismiss': 'alert',
            'aria-label': 'Close',
        })
        alert.append(dismiss_btn);
        $(this).prepend(alert);
    }
})(jQuery);