(function($){
"use strict";
$.fn.InitPicBoard = function(init_scale, max_height) {
    var that = this;

    this.has_descs = true;
    this.show_descs = false;
    this.force_expand = false;

    this.url = $(this).find('.sentence-pic-data url').text();

    var is_large_screen = $(window).width() >= 992;

    if (is_large_screen) {
        this.show_descs = true;
    }

    var pic = $(this).find('.sentence-pic');
    try {
        this.TopMindGraph = DrawMindGraph(pic, init_scale, max_height, this.has_descs, this.show_descs, false);
    }
    catch(e) {
        return;
    }
    this.has_descs = this.TopMindGraph.has_descs();
    this.show_descs = this.TopMindGraph.has_descs_shown();

    // $(this).addClass('theme-pure theme-red');
    var checks = $(this).find('.pic-board-options .form-check');
    for (var i = 0; i < checks.length; i++) {
        var check = checks[i];
        var input = $(check).find('input');
        var label = $(check).find('label');
        input.attr('id', input.attr('id') + '-' +getUniqueID());
        label.attr('for', input.attr('id'));
    }

    if (this.has_descs) {
        var descs_opt = $(this).find('.pic-board-options .option-show-descs');
        if (this.show_descs) {
            descs_opt.prop("checked", true);
        }
        descs_opt.parent().removeClass('d-none');
    }

    if (this.TopMindGraph.can_expand()) {
        $(this).find('.pic-board-options .option-expand-all').parent().removeClass('d-none');
    }
    
    if (is_large_screen) {
        $(that).find('.pic-board-editor').addClass('expanded');
    }

    this.ScrollToCenter = function() {
        if (pic[0].scrollWidth > pic[0].clientWidth) {
            pic.scrollLeft((pic[0].scrollWidth - pic[0].clientWidth)/2);
        }
        if (pic[0].scrollHeight > pic[0].clientHeight) {
            pic.scrollTop((pic[0].scrollHeight - pic[0].clientHeight)/2);
        }
    }
    
    this.ScrollToCenter();

    $(this).on('click', '.pic-board-options .form-check input, .pic-board-options .form-check label', function(e) {
        e.stopPropagation();
    });

    $(this).on('click', '.pic-editor-expand', function(e) {
        e.stopPropagation();
        e.preventDefault();
        var editor = $(that).find('.pic-board-editor');
        if (editor.hasClass('expanded')) {
            editor.removeClass('expanded');
        }
        else {
            editor.addClass('expanded');
        }
        if ($(this).hasClass('pic-editor-expand-arrow')) {
            $(this).find('i').toggleClass('fa-chevron-down').toggleClass('fa-chevron-up');
        }
    })

    $(this).on('click', '.pic-board-editor .pic-editor-theme', function(e) {
        e.stopPropagation();
        e.preventDefault();
        $(that).find('.pic-editor-theme .fa-check').addClass('d-none');
        $(this).find('.fa-check').removeClass('d-none');
        var value = $(this).attr('value');
        $(that).removeClass(that.theme).addClass(value);
        that.theme = value;
    })

    $(this).on('click', '.pic-board-editor .pic-editor-theme-more', function(e) {
        e.stopPropagation();
        e.preventDefault();
        if ($(this).hasClass('on')) {
            $(this).removeClass('on');
            $(that).find('.pic-board-editor-more-themes').addClass('d-none');
        }
        else {
            $(this).addClass('on');
            $(that).find('.pic-board-editor-more-themes').removeClass('d-none');
        }
    })
    
    $(this).on('click', '.pic-board-editor .pic-editor-size', function(e) {
        e.stopPropagation();
        e.preventDefault();
        if (!that.TopMindGraph) {
            return;
        }
        var value = $(this).attr('value');
        if (value == 'plus') {
            var cur_scale = that.TopMindGraph.get_scale();
            if (cur_scale >= 3) {
                return;
            }
            that.TopMindGraph.set_scale(cur_scale + 0.1);
        } else {
            var cur_scale = that.TopMindGraph.get_scale();
            if (cur_scale <= 0.5) {
                return;
            }
            that.TopMindGraph.set_scale(cur_scale - 0.1);
        }
    })
    
    $(this).on('click', '.pic-board-editor .pic-editor-qrcode', function(e) {
        e.stopPropagation();
        e.preventDefault();
        var wrap = $(this).find('.qrcode-wrap');
        if (!wrap.hasClass('d-none')) {
            wrap.addClass('d-none');
            return;
        }
        wrap.empty().removeClass('d-none');
        wrap.qrcode({
            render: 'image',
            size: 150,
            text: 'https://enpuz.com/' + that.url,
            radius: 0.5,
            fill: _ColorList[getRandomInt(0,_ColorList.length - 1)]});
    })
    
    $(this).on('change', '.pic-board-options .option-show-descs', function(e) {
        e.stopPropagation();
        e.preventDefault();
        var checked = $(this).prop('checked');
        pic.scrollLeft(0);
        if (checked) {
            that.TopMindGraph.show_descs();
        }
        else {
            that.TopMindGraph.hide_descs();
        }
        that.show_descs = that.TopMindGraph.has_descs_shown();
        that.ScrollToCenter();
    })

    $(this).on('change', '.pic-board-options .option-expand-all', function(e) {
        e.stopPropagation();
        e.preventDefault();
        var checked = $(this).prop('checked');
        pic.scrollLeft(0);
        if (checked) {
            that.force_expand = true;
            that.TopMindGraph = DrawMindGraph(pic, init_scale, max_height, that.has_descs, that.show_descs, that.force_expand);
        }
        else {
            that.force_expand = false;
            that.TopMindGraph = DrawMindGraph(pic, init_scale, max_height, that.has_descs, that.show_descs, that.force_expand);
        }
        that.ScrollToCenter();
    })
    
    $(this).on('click', '.pic-board-editor .pic-editor-camera', function(e) {
        e.stopPropagation();
        e.preventDefault();
        var pic_share_modal = new bootstrap.Modal('#SharePicModal', {});
        pic_share_modal.show();
        
        var cur_svg = $(that).find('svg.ming-svg-screen');
        var full_screen_svg_wrap = $('#SharePicModal .sent-svg');
        if (!that.theme) {
            that.theme = 'theme-' + $('body').attr('theme');
        }
        full_screen_svg_wrap.attr('class', 'sent-svg pic-board ' + that.theme);
        full_screen_svg_wrap.html(cur_svg[0].outerHTML);

        var svg = full_screen_svg_wrap.find('svg.ming-svg-screen');
        var pic_body = full_screen_svg_wrap.find('.sentence-pic-body');
        if (!pic_body.hasClass('style-expanded')) {
            pic_body.addClass('style-expanded');
            full_screen_svg_wrap.find('*').each(function(){
                for (var i = 0; i < SVG_STYLES.length; i++) {
                    var style = SVG_STYLES[i];
                    var value = $(this).css(style);
                    if (value == 'auto') {
                        continue;
                    }
                    $(this).css(style, value);
                }
            })
        }
        var svghtml = svg.prop('outerHTML');
        var img = new Image();
        var imgsrc = 'data:image/svg+xml;base64,' + window.btoa(unescape(encodeURIComponent(svghtml)));
        img.src = imgsrc;
        img.onload = function () {
            var canvas = document.createElement('canvas');
            var width = svg.width();
            var height = svg.height();
            canvas.width = width * 4 + 200;
            canvas.height = height * 4 + 200;
            if (canvas.width * canvas.height > 16777216) {
                // ios 限制
                var limit_rate = Math.sqrt(canvas.width * canvas.height / 16777216);
                width = (canvas.width / limit_rate - 200) / 4 * 0.98;
                height = (canvas.height / limit_rate - 200) / 4 * 0.98;
                canvas.width = width * 4 + 200;
                canvas.height = height * 4 + 200;
            }
            var context = canvas.getContext('2d');
            context.fillStyle = full_screen_svg_wrap.find('svg.ming-svg-screen').css('background-color');
            context.fillRect(0, 0, canvas.width, canvas.height);
            context.drawImage(img, 100, 100, width * 4, height * 4);
            var url = canvas.toDataURL('image/png');
            var rate = canvas.width / canvas.height;

            var real_width = $(window).width() - 40;
            if (canvas.width * 1.2 < real_width) {
                real_width = canvas.width * 1.2;
            }
            var real_height = real_width / rate;

            $('#SharePicModal .sent').removeClass('d-none');
            $('#SharePicModal .sent')
                .attr('src', url)
                .width(real_width)
                .height(real_height);
            var query = $('#ResultData result-data query').text();
            $('#SharePicModal h5 .right').text(query);
        }
    })

    return this;
}
})(jQuery);